import { Function1, SideEffect2 } from "@reactive-js/core/lib/functions";
import { IOSourceLike } from "@reactive-js/core/lib/io";
import { CacheDirective } from "./cacheDirective";
import { EntityTag } from "./entityTag";
import { HttpContentEncoding } from "./httpContentInfo";
import { HttpHeaders } from "./httpHeaders";
import { HttpMessage, URILike } from "./httpMessage";
import { MediaRange } from "./httpPreferences";
import { HttpRequestPreconditions } from "./httpRequestPreconditions";
import { HttpResponse } from "./httpResponse";
import { MediaType } from "./mediaType";
export declare const enum HttpMethod {
    GET = "GET",
    HEAD = "HEAD",
    POST = "POST",
    PUT = "PUT",
    DELETE = "DELETE"
}
export declare type HttpRequest<T> = HttpMessage<T> & {
    readonly expectContinue: boolean;
    readonly method: HttpMethod;
    readonly preconditions?: HttpRequestPreconditions;
    readonly uri: URILike;
    readonly httpVersionMajor: number;
    readonly httpVersionMinor: number;
};
export declare const createHttpRequest: <T>({ body, cacheControl, contentInfo, expectContinue, headers, httpVersionMajor, httpVersionMinor, method, preconditions, preferences, uri, ...rest }: {
    body: T;
    cacheControl?: readonly (string | CacheDirective)[] | undefined;
    contentInfo?: {
        contentEncodings?: readonly HttpContentEncoding[] | undefined;
        contentLength?: number | undefined;
        contentType: string | MediaType;
    } | undefined;
    expectContinue?: boolean | undefined;
    headers?: import("@reactive-js/core/lib/option").Option<HttpHeaders>;
    httpVersionMajor?: number | undefined;
    httpVersionMinor?: number | undefined;
    method: HttpMethod;
    preconditions?: {
        ifMatch?: "*" | readonly (string | EntityTag)[] | undefined;
        ifModifiedSince?: string | number | Date | undefined;
        ifNoneMatch?: "*" | readonly (string | EntityTag)[] | undefined;
        ifUnmodifiedSince?: string | number | Date | undefined;
        ifRange?: string | number | Date | EntityTag | undefined;
    } | undefined;
    preferences?: {
        acceptedCharsets?: import("@reactive-js/core/lib/option").Option<readonly string[]>;
        acceptedEncodings?: readonly HttpContentEncoding[] | undefined;
        acceptedLanguages?: import("@reactive-js/core/lib/option").Option<readonly string[]>;
        acceptedMediaRanges?: readonly (string | MediaRange)[] | undefined;
    } | undefined;
    uri: string | URILike;
}) => HttpRequest<T>;
export declare const createRedirectHttpRequest: <THttpRequest extends HttpRequest<TReq>, TReq>(request: THttpRequest, response: HttpResponse<unknown>) => THttpRequest;
export declare const writeHttpRequestHeaders: <T>(request: HttpRequest<T>, writeHeader: SideEffect2<string, string>) => void;
export declare const httpRequestToUntypedHeaders: (request: HttpRequest<unknown>) => {
    [key: string]: string;
};
export declare const encodeHttpRequestWithUtf8: Function1<HttpRequest<string>, HttpRequest<Uint8Array>>;
export declare const decodeHttpRequestWithCharset: Function1<HttpRequest<Uint8Array>, HttpRequest<string>>;
export declare const toIOSourceHttpRequest: <TBody>(req: HttpRequest<TBody>) => HttpRequest<IOSourceLike<TBody>>;
export declare const decodeHttpRequestContent: (decoderProvider: {
    [key: string]: Function1<IOSourceLike<Uint8Array>, IOSourceLike<Uint8Array>>;
}) => Function1<HttpRequest<IOSourceLike<Uint8Array>>, HttpRequest<IOSourceLike<Uint8Array>>>;
