"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
const functions_1 = require("@reactive-js/core/lib/functions");
const option_1 = require("@reactive-js/core/lib/option");
const readonlyArray_1 = require("@reactive-js/core/lib/readonlyArray");
const cacheDirective_1 = require("./cacheDirective");
const httpContentInfo_1 = require("./httpContentInfo");
const httpHeaders_1 = require("./httpHeaders");
const httpMessage_1 = require("./httpMessage");
const httpPreferences_1 = require("./httpPreferences");
const httpRequestPreconditions_1 = require("./httpRequestPreconditions");
const httpResponse_1 = require("./httpResponse");
exports.createHttpRequest = (_a) => {
    var { body, cacheControl, contentInfo, expectContinue = false, headers = {}, httpVersionMajor = 1, httpVersionMinor = 1, method, preconditions, preferences, uri } = _a, rest = __rest(_a, ["body", "cacheControl", "contentInfo", "expectContinue", "headers", "httpVersionMajor", "httpVersionMinor", "method", "preconditions", "preferences", "uri"]);
    return (Object.assign(Object.assign({}, rest), { body, cacheControl: functions_1.pipe(cacheControl !== null && cacheControl !== void 0 ? cacheControl : [], readonlyArray_1.map(cc => (typeof cc === "string" ? cacheDirective_1.parseCacheDirectiveOrThrow(cc) : cc))), contentInfo: option_1.isSome(contentInfo)
            ? httpContentInfo_1.createHttpContentInfo(contentInfo)
            : httpContentInfo_1.parseHttpContentInfoFromHeaders(headers), expectContinue: expectContinue, headers: httpHeaders_1.filterHeaders(headers), httpVersionMajor: httpVersionMajor, httpVersionMinor: httpVersionMinor, method, preconditions: option_1.isSome(preconditions)
            ? httpRequestPreconditions_1.createHttpRequestPreconditions(preconditions)
            : httpRequestPreconditions_1.parseHttpRequestPreconditionsFromHeaders(headers), preferences: option_1.isSome(preferences)
            ? httpPreferences_1.createHttpPreferences(preferences)
            : httpPreferences_1.parseHttpPreferencesFromHeaders(headers), uri: typeof uri === "string" ? new URL(uri) : uri }));
};
exports.createRedirectHttpRequest = (request, response) => {
    const { contentInfo, method } = request;
    const { location, statusCode } = response;
    const redirectToGet = statusCode === 303 ||
        ((statusCode === 301 ||
            302 === 302) &&
            method === "POST");
    return Object.assign(Object.assign({}, request), { content: redirectToGet ? option_1.none : contentInfo, method: redirectToGet ? "GET" : method, uri: location });
};
exports.writeHttpRequestHeaders = (request, writeHeader) => {
    const { expectContinue, preconditions } = request;
    if (expectContinue) {
        writeHeader("Expect", "100-continue");
    }
    if (option_1.isSome(preconditions)) {
        httpRequestPreconditions_1.writeHttpRequestPreconditionsHeaders(preconditions, writeHeader);
    }
    httpMessage_1.writeHttpMessageHeaders(request, writeHeader);
};
exports.httpRequestToUntypedHeaders = (request) => {
    const headers = {};
    exports.writeHttpRequestHeaders(request, (header, value) => (headers[header] = value));
    return headers;
};
exports.encodeHttpRequestWithUtf8 = httpMessage_1.encodeHttpMessageWithUtf8;
exports.decodeHttpRequestWithCharset = httpMessage_1.decodeHttpMessageWithCharset;
exports.toIOSourceHttpRequest = (req) => httpMessage_1.toIOSourceHttpMessage(req);
exports.decodeHttpRequestContent = (decoderProvider) => req => {
    const { body, contentInfo } = req, rest = __rest(req, ["body", "contentInfo"]);
    if (option_1.isSome(contentInfo) && contentInfo.contentEncodings.length > 0) {
        const newBody = functions_1.pipe(contentInfo.contentEncodings, readonlyArray_1.map(encoding => {
            const decoder = decoderProvider[encoding];
            if (option_1.isNone(decoder)) {
                throw httpResponse_1.createHttpResponse({
                    statusCode: 415,
                    body: option_1.none,
                });
            }
            return decoder;
        }), readonlyArray_1.reduceRight((acc, decoder) => decoder(acc), functions_1.returns(body)));
        return Object.assign(Object.assign({}, rest), { contentInfo: {
                contentType: contentInfo.contentType,
                contentEncodings: [],
                contentLength: -1,
            }, body: newBody });
    }
    else {
        return req;
    }
};
