import { Function1, SideEffect2 } from "@reactive-js/core/lib/functions";
import { IOSourceLike } from "@reactive-js/core/lib/io";
import { CacheDirective } from "./cacheDirective";
import { EntityTag } from "./entityTag";
import { HttpContentEncoding } from "./httpContentInfo";
import { HttpDateTime } from "./httpDateTime";
import { HttpHeaders } from "./httpHeaders";
import { HttpMessage, URILike } from "./httpMessage";
import { MediaRange } from "./httpPreferences";
import { HttpRequest } from "./httpRequest";
import { MediaType } from "./mediaType";
export declare const enum HttpStatusCode {
    Continue = 100,
    SwitchingProtocols = 101,
    Processing = 102,
    OK = 200,
    Created = 201,
    Accepted = 202,
    NonAuthoritativeInformation = 203,
    NoContent = 204,
    ResetContent = 205,
    PartialContent = 206,
    MultiStatus = 207,
    AlreadyReported = 208,
    IMUsed = 226,
    MultipleChoices = 300,
    MovedPermanently = 301,
    Found = 302,
    SeeOther = 303,
    NotModified = 304,
    UseProxy = 305,
    TemporaryRedirect = 307,
    PermanentRedirect = 308,
    BadRequest = 400,
    Unauthorized = 401,
    Forbidden = 403,
    NotFound = 404,
    MethodNotAllowed = 405,
    NotAcceptable = 406,
    ProxyAuthenticationRequired = 407,
    RequestTimeout = 408,
    Conflict = 409,
    Gone = 410,
    LengthRequired = 411,
    PreconditionFailed = 412,
    RequestEntityTooLarge = 413,
    RequestURITooLong = 414,
    UnsupportedMediaType = 415,
    RequestedRangeNotSatisfiable = 416,
    ExpectationFailed = 417,
    UnprocessableEntity = 422,
    Locked = 423,
    FailedDependency = 424,
    UpgradeRequired = 426,
    PreconditionRequired = 428,
    TooManyRequests = 429,
    RequestHeaderFieldsTooLarge = 431,
    UnavailableForLegalReasons = 451,
    InternalServerError = 500,
    NotImplemented = 501,
    BadGateway = 502,
    ServiceUnavailable = 503,
    GatewayTimeout = 504,
    HTTPVersionNotSupported = 505,
    VariantAlsoNegotiates = 506,
    InsufficientStorage = 507,
    LoopDetected = 508,
    NotExtended = 510,
    NetworkAuthenticationRequired = 511
}
export declare type HttpResponse<T> = HttpMessage<T> & {
    readonly etag?: EntityTag;
    readonly expires?: HttpDateTime;
    readonly lastModified?: HttpDateTime;
    readonly location?: URILike;
    readonly statusCode: HttpStatusCode;
    readonly vary: readonly string[];
};
export declare const createHttpResponse: <T>({ body, cacheControl, contentInfo, etag, expires, headers, lastModified, location, preferences, statusCode, vary, ...rest }: {
    body: T;
    cacheControl?: readonly (string | CacheDirective)[] | undefined;
    contentInfo?: {
        contentEncodings?: readonly HttpContentEncoding[] | undefined;
        contentLength?: number | undefined;
        contentType: string | MediaType;
    } | undefined;
    etag?: string | EntityTag | undefined;
    expires?: number | undefined;
    headers?: import("@reactive-js/core/lib/option").Option<HttpHeaders>;
    lastModified?: string | number | Date | undefined;
    location?: string | URILike | undefined;
    preferences?: {
        acceptedCharsets?: import("@reactive-js/core/lib/option").Option<readonly string[]>;
        acceptedEncodings?: readonly HttpContentEncoding[] | undefined;
        acceptedLanguages?: import("@reactive-js/core/lib/option").Option<readonly string[]>;
        acceptedMediaRanges?: readonly (string | MediaRange)[] | undefined;
    } | undefined;
    statusCode: HttpStatusCode;
    vary?: import("@reactive-js/core/lib/option").Option<readonly string[]>;
}) => HttpResponse<T>;
export declare const parseHttpResponseFromHeaders: <T>(statusCode: number, headers: HttpHeaders, body: T) => HttpResponse<T>;
export declare const writeHttpResponseHeaders: <T>(response: HttpResponse<T>, writeHeader: SideEffect2<string, string>) => void;
export declare const checkIfNotModified: <T>({ cacheControl, method, preconditions, }: HttpRequest<unknown>) => Function1<HttpResponse<T>, HttpResponse<T>>;
export declare const encodeHttpResponseWithUtf8: Function1<HttpResponse<string>, HttpResponse<Uint8Array>>;
export declare const decodeHttpResponseWithCharset: Function1<HttpResponse<Uint8Array>, HttpResponse<string>>;
export declare const toIOSourceHttpResponse: <TBody>(resp: HttpResponse<TBody>) => HttpResponse<IOSourceLike<TBody>>;
export declare const decodeHttpResponseContent: (decoderProvider: {
    [key: string]: Function1<IOSourceLike<Uint8Array>, IOSourceLike<Uint8Array>>;
}) => Function1<HttpResponse<IOSourceLike<Uint8Array>>, HttpResponse<IOSourceLike<Uint8Array>>>;
export declare const encodeHttpResponseContent: (encoderProvider: {
    [key: string]: Function1<IOSourceLike<Uint8Array>, IOSourceLike<Uint8Array>>;
}, db?: {
    [key: string]: {
        compressible?: boolean | undefined;
    };
}) => (request: HttpRequest<unknown>) => Function1<HttpResponse<IOSourceLike<Uint8Array>>, HttpResponse<IOSourceLike<Uint8Array>>>;
export declare const createHttpErrorResponse: (e: unknown) => HttpResponse<unknown>;
