"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
const functions_1 = require("@reactive-js/core/lib/functions");
const option_1 = require("@reactive-js/core/lib/option");
const readonlyArray_1 = require("@reactive-js/core/lib/readonlyArray");
const cacheDirective_1 = require("./cacheDirective");
const httpContentInfo_1 = require("./httpContentInfo");
const httpHeaders_1 = require("./httpHeaders");
const httpPreferences_1 = require("./httpPreferences");
const httpRequestPreconditions_1 = require("./httpRequestPreconditions");
const createSegments = (path) => {
    const root = { name: "" };
    let acc = root;
    for (const name of path.split("/")) {
        const child = { name };
        acc.child = child;
        acc = child;
    }
    return root;
};
const serializeSegments = (segment) => {
    let result = segment.name;
    while (option_1.isSome(segment.child)) {
        segment = segment.child;
        result += `/${segment.name}`;
    }
    return result;
};
const emptyRouter = {
    name: "",
    children: {},
};
const addHandler = (router, { name, child }, handler) => {
    var _a;
    if (option_1.isNone(child)) {
        return {
            name,
            handler,
            children: router.children,
        };
    }
    else {
        const childName = child.name.startsWith(":") ? ":" : child.name;
        const childRouter = (_a = router.children[childName]) !== null && _a !== void 0 ? _a : emptyRouter;
        const newChildRouter = addHandler(childRouter, child, handler);
        return {
            name,
            handler: router.handler,
            children: Object.assign(Object.assign({}, router.children), { [childName]: newChildRouter }),
        };
    }
};
const findHandler = (router, segment, params) => {
    const { child } = segment;
    const { handler } = router;
    if (option_1.isNone(child) && option_1.isSome(handler)) {
        return [handler, params];
    }
    if (option_1.isNone(child)) {
        return option_1.none;
    }
    const nameRouter = router.children[child.name];
    const nameRouterResult = option_1.isSome(nameRouter)
        ? findHandler(nameRouter, child, params)
        : option_1.none;
    if (option_1.isSome(nameRouterResult)) {
        return nameRouterResult;
    }
    const paramRouter = router.children[":"];
    const paramRouterResult = option_1.isSome(paramRouter)
        ? findHandler(paramRouter, child, Object.assign(Object.assign({}, params), { [paramRouter.name]: child.name }))
        : option_1.none;
    if (option_1.isSome(paramRouterResult)) {
        return paramRouterResult;
    }
    const globRouter = router.children["*"];
    const globRouterHandler = globRouter === null || globRouter === void 0 ? void 0 : globRouter.handler;
    if (option_1.isSome(globRouterHandler)) {
        const newParams = Object.assign(Object.assign({}, params), { ["*"]: serializeSegments(child) });
        return [globRouterHandler, newParams];
    }
    return option_1.none;
};
exports.createRoutingHttpServer = (routes, notFoundHandler) => {
    const router = functions_1.pipe(routes, readonlyArray_1.fromObject(), readonlyArray_1.reduce((acc, [path, handler]) => addHandler(acc, createSegments(path), handler), functions_1.returns(emptyRouter)));
    return (request) => {
        const segments = createSegments(request.uri.pathname);
        const result = findHandler(router, segments, {});
        if (option_1.isSome(result)) {
            const [handler, params] = result;
            const requestWithParams = Object.assign(Object.assign({}, request), { params });
            return handler(requestWithParams);
        }
        else {
            return notFoundHandler(request);
        }
    };
};
const parseURIFromHeaders = (protocol, path, httpVersionMajor, headers) => {
    const forwardedProtocol = httpHeaders_1.getHeaderValue(headers, "X-Forwarded-Proto");
    const uriProtocol = option_1.isSome(forwardedProtocol)
        ? forwardedProtocol.split(/\s*,\s*/, 1)[0]
        : protocol;
    const forwardedHost = httpHeaders_1.getHeaderValue(headers, "X-Forwarded-Host");
    const http2Authority = headers[":authority"];
    const http1Host = httpHeaders_1.getHeaderValue(headers, "Host");
    const unfilteredHost = option_1.isSome(forwardedHost)
        ? forwardedHost
        : option_1.isSome(http2Authority) && httpVersionMajor >= 2
            ? http2Authority
            : option_1.isSome(http1Host)
                ? http1Host
                : "";
    const host = unfilteredHost.split(/\s*,\s*/, 1)[0];
    return new URL(`${uriProtocol}://${host}${path !== null && path !== void 0 ? path : ""}`);
};
exports.parseHttpServerRequestFromHeaders = ({ method, path, headers, httpVersionMajor, httpVersionMinor, isTransportSecure, body, }) => {
    const cacheControl = cacheDirective_1.parseCacheControlFromHeaders(headers);
    const contentInfo = httpContentInfo_1.parseHttpContentInfoFromHeaders(headers);
    const rawExpectHeader = httpHeaders_1.getHeaderValue(headers, "Expect");
    const expectContinue = rawExpectHeader === "100-continue";
    const preconditions = httpRequestPreconditions_1.parseHttpRequestPreconditionsFromHeaders(headers);
    const preferences = httpPreferences_1.parseHttpPreferencesFromHeaders(headers);
    const protocol = isTransportSecure ? "https" : "http";
    const uri = parseURIFromHeaders(protocol, path, httpVersionMajor, headers);
    return {
        body,
        cacheControl,
        contentInfo,
        expectContinue,
        headers,
        httpVersionMajor,
        httpVersionMinor,
        isTransportSecure,
        method,
        preconditions,
        preferences,
        uri,
    };
};
exports.disallowProtocolAndHostForwarding = () => request => {
    const { httpVersionMajor, headers: oldHeaders, isTransportSecure, uri: oldUri, } = request;
    const { "x-forwarded-proto": xForwardedProto, "x-forwarded-host": xForwardedHost } = oldHeaders, headers = __rest(oldHeaders, ["x-forwarded-proto", "x-forwarded-host"]);
    const protocol = isTransportSecure ? "https" : "http";
    if (option_1.isNone(xForwardedProto) && option_1.isNone(xForwardedHost)) {
        return request;
    }
    else {
        const path = oldUri.pathname;
        const uri = parseURIFromHeaders(protocol, path, httpVersionMajor, headers);
        return Object.assign(Object.assign({}, request), { uri,
            headers });
    }
};
